datablock ItemData(satchelChargeItem)
{
	shapeFile = "./satchelCharge.dts";
	uiName = "Satchel Charge";
	doColorShift = true;
	colorShiftColor = "0.3 0.7 0.3";
	iconName = "./icon_charge";
	
	rotate = false;
	mass = 1;
	density = 0.2;
	elasticity = 0.2;
	friction = 0.6;
	emap = true;
	
	canDrop = true;
	
	image = satchelChargeImage;
};

datablock ShapeBaseImageData(satchelChargeImage)
{
	shapeFile = "./satchelCharge.dts";
	
	offset = "-0.15 0.1 0.15";
	rotation = eulerToMatrix("90 0 0");
	
	armReady = true;
	minShotTime = 700;
	
	//projectile = snarkProjectile;
	doColorShift = true;
	colorShiftColor = satchelChargeItem.colorShiftColor;
	
	stateName[0]                     	= "Activate";
	stateTimeoutValue[0]            	= 0.15;
	stateTransitionOnTimeout[0]       	= "Ready";
	stateSound[0]						= weaponSwitchSound;

	stateName[1]                     	= "Ready";
	stateTransitionOnTriggerDown[1]  	= "Fire";
	stateAllowImageChange[1]         	= true;
	stateSequence[1]				 	= "Ready";

	stateName[2]                    	= "Fire";
	stateTransitionOnTimeout[2]     	= "Smoke";
	stateTimeoutValue[2]            	= 0.14;
	stateFire[2]                    	= true;
	stateAllowImageChange[2]        	= false;
	stateSequence[2]                	= "Fire";
	stateScript[2]                  	= "onFire";
	stateWaitForTimeout[2]				= true;
	stateEmitter[2]						= "";
	stateEmitterTime[2]					= 0.05;
	stateEmitterNode[2]					= "muzzleNode";
	stateSound[2]						= "";
	stateEjectShell[2]       			= true;

	stateName[3] 						= "Smoke";
	stateEmitter[3]						= "";
	stateSequence[3]                	= "Fire";
	stateEmitterTime[3]					= 0.05;
	stateEmitterNode[3]					= "muzzleNode";
	stateTimeoutValue[3]            	= 0.5;
	stateTransitionOnTimeout[3]     	= "Reload";

	stateName[4]						= "Reload";
	stateSequence[4]                	= "Fire";
	stateTransitionOnTriggerUp[4]     	= "Ready";
};

function satchelChargeImage::onFire(%this, %obj, %slot)
{
	// init vars
	%client = %obj.client;
	%player = %obj;
	%handPos = %player.getMuzzlePoint(0);
	%vector = %player.getEyeVector();
	%velocity = vectorScale(%player.getMuzzleVector(0), 16);
	%rotation = getWords(%player.getTransform(), 3, 6);
	
	// create charge
	%charge = new Item()
	{
		dataBlock = satchelChargeItem;
		client = %client;
	};
	missionCleanup.add(%charge);
	%charge.canPickup = false;
	%charge.setTransform(%handPos SPC %rotation);
	%charge.setVelocity(%velocity);
	%charge.age = getSimTime();
	%charge.player = %player;
	%charge.schedule(600000, delete);
	
	// Add satcharge to our simsets
	if(!isObject(%player.satchelChargeSet))
		%player.satchelChargeSet = new SimSet();
		
	if(!isObject($GlobalSatChargeSet))
		$GlobalSatChargeSet = new SimSet();
	
	%player.satchelChargeSet.add(%charge);
	$GlobalSatChargeSet.add(%charge);
	
	// Handle detonator
	for(%i = 0; %i < %player.getDatablock().maxTools; %i++)
	{
		%item = %player.tool[%i];
		if(%item == nameToID("satchelDetonatorItem"))
		{
			%foundDetonator = true;
		}
	}
	
	// Removal/addition of satcharges and detonators
	if(!$Pref::SatCharge_InfUses)
	{
		%currTool = %player.currTool;
		%player.tool[%currTool] = 0;
		%player.weaponCount--;
		messageClient(%client, 'MsgItemPickup', '', %currTool, 0);
		serverCmdUnUseTool(%client);
		
		if(!%foundDetonator)
		{
			%player.tool[%currTool] = nameToID("satchelDetonatorItem");
			%player.weaponCount++;
			messageClient(%client, 'MsgItemPickup', '', %currTool, nameToID("satchelDetonatorItem"));
			serverCmdUseTool(%client);
		}
	}
	else
	{
		if(!%foundDetonator)
		{
			for(%i = 0; %i < %player.getDatablock().maxTools; %i++)
			{
				%item = %obj.tool[%i];
				if(%item $= 0 || %item $= "")
				{
					%freeSlot = 1;
					%freeSlotIndex = %i;
					break;
				}
			}

			if(%freeSlot)
			{
				%player.tool[%freeSlotIndex] = nameToID("satchelDetonatorItem");
				%player.weaponCount++;
				messageClient(%client, 'MsgItemPickup', '', %freeSlotIndex, nameToID("satchelDetonatorItem"));
				serverCmdUseTool(%client);
			}
		}
		
		// remove oldest from set if too many
		while(%player.satchelChargeSet.getCount() > $Pref::SatCharge_MaxAmt)
		{
			%oldtime = getSimTime();
			%oldest = 0;
			for(%i = 0; %i < %player.satchelChargeSet.getCount(); %i++)
			{
				%o = %player.satchelChargeSet.getObject(%i);
				if(%o.age < %oldtime)
				{
					%oldtime = %o.age;
					%oldest = %i;
				}
			}
			%o = %player.satchelChargeSet.getObject(%oldest);
			
			if(isObject(%o))
			{
				%o.delete();
			}
		}
	}
}

function Player::DetonateSatchelCharges(%player)
{
	// init vars
	%set = %player.satchelChargeSet;
	%client = %player.client;
	
	// no set, return
	if(!isObject(%set))
		return;
	
	// keep detonating charges in the set until there are none left
	while(%set.getCount())
	{
		%charge = %set.getObject(%i);
		%position = %charge.getPosition();
		
		%proj = new Projectile()
		{
			dataBlock = tankShellProjectile;
			initialPosition = %position;
			client = %client;
		};
		%proj.setScale($Pref::SatCharge_Scale SPC $Pref::SatCharge_Scale SPC $Pref::SatCharge_Scale);
		MissionCleanup.add(%proj);
		%proj.explode();
		
		if(isObject(%charge))
			%charge.delete();
	}
}

package SatchelChargePackagae
{
	// remove on death/disconnect
	function Armor::onDisabled(%this, %player, %state)
	{
		if(isObject(%player.satchelChargeSet))
		{
			%set = %player.satchelChargeSet;
			while(%set.getCount())
			{
				%charge = %set.getObject(%i);
				
				if(isObject(%charge))
					%charge.delete();
			}
		}
		Parent::onDisabled(%this, %player, %state);
	}
	function Armor::onRemove(%this, %player)
	{
		if(isObject(%player.satchelChargeSet))
		{
			%set = %player.satchelChargeSet;
			while(%set.getCount())
			{
				%charge = %set.getObject(%i);
				
				if(isObject(%charge))
					%charge.delete();
			}
		}
		Parent::onRemove(%this, %player);
	}

	// can hold multiple
	function Armor::onCollision(%this, %obj, %col, %a, %b, %c, %d, %e, %f)
	{
		if(%col.dataBlock $= "satchelChargeItem" && %col.canPickup && !$Pref::SatCharge_InfUses)
		{
			for(%i=0;%i<%this.maxTools;%i++)
			{
				%item = %obj.tool[%i];
				if(%item $= 0 || %item $= "")
				{
					%freeSlot = 1;
					break;
				}
			}

			if(%freeSlot)
			{
				%obj.pickup(%col);
				return;
			}
		}
		Parent::onCollision(%this, %obj, %col, %a, %b, %c, %d, %e, %f);
	}

};
ActivatePackage(SatchelChargePackagae);